@@ -17,32 +17,25 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
-from oslo_log import log as logging
-
 from congress.api import api_utils
 from congress.api import base
 from congress.api import webservice
 from congress import exception
 
 
-LOG = logging.getLogger(__name__)
-
-
 class StatusModel(base.APIModel):
     """Model for handling API requests about Statuses."""
 
     # Note(thread-safety): blocking function
     def get_item(self, id_, params, context=None):
-        """Retrieve item with id id_ from model.
+        """Retrieve item with id id\_ from model.
 
-        Args:
-            id_: The ID of the item to retrieve
-            params: A dict-like object containing parameters
+        :param: id\_: The ID of the item to retrieve
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            context: Key-values providing frame of reference of request
+        :param: context: Key-values providing frame of reference of request
 
-        Returns:
-             The matching item or None if item with id_ does not exist.
+        :returns: The matching item or None if item with id\_ does not exist.
         """
         # Note(thread-safety): blocking call
         caller, source_id = api_utils.get_id_from_context(context)
