@@ -23,44 +23,36 @@ from congress.api import webservice
 from congress import exception
 
 
-def d6service(name, keys, inbox, datapath, args):
-    return DatasourceDriverModel(name, keys, inbox=inbox,
-                                 dataPath=datapath, **args)
-
-
 class DatasourceDriverModel(base.APIModel):
     """Model for handling API requests about DatasourceDriver."""
 
     def get_items(self, params, context=None):
         """Get items in model.
 
-        Args:
-            params: A dict-like object containing parameters
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            context: Key-values providing frame of reference of request
+        :param: context: Key-values providing frame of reference of request
 
-        Returns: A dict containing at least a 'results' key whose value is
+        :returns: A dict containing at least a 'results' key whose value is
                  a list of items in the model.  Additional keys set in the
                  dict will also be rendered for the user.
         """
         drivers = self.bus.get_drivers_info()
         fields = ['id', 'description']
         results = [self.bus.make_datasource_dict(
-                   drivers[driver], fields=fields)
+                   driver, fields=fields)
                    for driver in drivers]
         return {"results": results}
 
     def get_item(self, id_, params, context=None):
-        """Retrieve item with id id_ from model.
+        """Retrieve item with id id\_ from model.
 
-        Args:
-            id_: The ID of the item to retrieve
-            params: A dict-like object containing parameters
+        :param: id\_: The ID of the item to retrieve
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            context: Key-values providing frame of reference of request
+        :param: context: Key-values providing frame of reference of request
 
-        Returns:
-             The matching item or None if item with id_ does not exist.
+        :returns: The matching item or None if item with id\_ does not exist.
         """
         datasource = context.get('driver_id')
         try:
