@@ -18,13 +18,14 @@ from __future__ import division
 from __future__ import absolute_import
 
 
-from oslo_log import log as logging
+from oslo_db import exception as oslo_db_exc
 import sqlalchemy as sa
 from sqlalchemy.orm import exc as db_exc
 
+
 from congress.db import api as db
 from congress.db import model_base
-
+from congress.db import utils as db_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -33,7 +34,7 @@ class Policy(model_base.BASE, model_base.HasId, model_base.HasAudit):
     __tablename__ = 'policies'
 
     # name is a human-readable string, so it can be referenced in policy
-    name = sa.Column(sa.Text(), nullable=False)
+    name = sa.Column(sa.String(255), nullable=False, unique=True)
     abbreviation = sa.Column(sa.String(5), nullable=False)
     description = sa.Column(sa.Text(), nullable=False)
     owner = sa.Column(sa.Text(), nullable=False)
@@ -60,16 +61,64 @@ class Policy(model_base.BASE, model_base.HasId, model_base.HasAudit):
         return d
 
 
+class PolicyDeleted(model_base.BASE, model_base.HasId, model_base.HasAudit):
+    __tablename__ = 'policiesdeleted'
+
+    # name is a human-readable string, so it can be referenced in policy
+    name = sa.Column(sa.String(255), nullable=False)
+    abbreviation = sa.Column(sa.String(5), nullable=False)
+    description = sa.Column(sa.Text(), nullable=False)
+    owner = sa.Column(sa.Text(), nullable=False)
+    kind = sa.Column(sa.Text(), nullable=False)
+
+    # overwrite some columns from HasAudit to stop value auto-generation
+    created_at = sa.Column(sa.DateTime, nullable=False)
+    updated_at = sa.Column(sa.DateTime, nullable=True)
+
+    def __init__(self, policy_obj):
+        '''Initialize a PolicyDeleted object by copying a Policy object.
+
+        :param: policy_obj: a Policy object
+        '''
+        self.id = policy_obj.id
+        self.name = policy_obj.name
+        self.abbreviation = policy_obj.abbreviation
+        self.description = policy_obj.description
+        self.owner = policy_obj.owner
+        self.kind = policy_obj.kind
+        self.deleted = policy_obj.deleted
+        self.created_at = policy_obj.created_at
+        self.updated_at = policy_obj.updated_at
+
+
+@db_utils.retry_on_db_error
 def add_policy(id_, name, abbreviation, description, owner, kind,
                deleted=False, session=None):
-    session = session or db.get_session()
-    with session.begin(subtransactions=True):
-        policy = Policy(id_, name, abbreviation, description, owner,
-                        kind, deleted)
-        session.add(policy)
-    return policy
+    if session:
+        # IMPORTANT: if session provided, do not interrupt existing transaction
+        # with BEGIN which can drop db locks and change desired transaction
+        # boundaries for proper commit and rollback
+        try:
+            policy = Policy(id_, name, abbreviation, description, owner,
+                            kind, deleted)
+            session.add(policy)
+            return policy
+        except oslo_db_exc.DBDuplicateEntry:
+            raise KeyError("Policy with name %s already exists" % name)
+
+    # else
+    session = db.get_session()
+    try:
+        with session.begin(subtransactions=True):
+            policy = Policy(id_, name, abbreviation, description, owner,
+                            kind, deleted)
+            session.add(policy)
+            return policy
+    except oslo_db_exc.DBDuplicateEntry:
+        raise KeyError("Policy with name %s already exists" % name)
 
 
+@db_utils.retry_on_db_error
 def delete_policy(id_, session=None):
     session = session or db.get_session()
     with session.begin(subtransactions=True):
@@ -77,12 +126,19 @@ def delete_policy(id_, session=None):
         policy = get_policy_by_id(id_, session=session)
         for rule in get_policy_rules(policy.name, session=session):
             delete_policy_rule(rule.id, session=session)
-        # delete the policy
-        return (session.query(Policy).
-                filter(Policy.id == id_).
-                soft_delete())
+
+        policy_deleted = PolicyDeleted(policy)
+        session.add(policy_deleted)
+
+        # hard delete policy in Policy table
+        session.query(Policy).filter(Policy.id == id_).delete()
+
+        # soft delete policy in PolicyDeleted table
+        return session.query(PolicyDeleted).filter(
+            PolicyDeleted.id == id_).soft_delete()
 
 
+@db_utils.retry_on_db_error
 def get_policy_by_id(id_, session=None, deleted=False):
     session = session or db.get_session()
     try:
@@ -94,6 +150,7 @@ def get_policy_by_id(id_, session=None, deleted=False):
         pass
 
 
+@db_utils.retry_on_db_error
 def get_policy_by_name(name, session=None, deleted=False):
     session = session or db.get_session()
     try:
@@ -105,6 +162,7 @@ def get_policy_by_name(name, session=None, deleted=False):
         pass
 
 
+@db_utils.retry_on_db_error
 def get_policy(name_or_id, session=None, deleted=False):
     # Try to retrieve policy either by id or name
     db_object = (get_policy_by_id(name_or_id, session, deleted) or
@@ -114,6 +172,7 @@ def get_policy(name_or_id, session=None, deleted=False):
     return db_object
 
 
+@db_utils.retry_on_db_error
 def get_policies(session=None, deleted=False):
     session = session or db.get_session()
     return (session.query(Policy).
@@ -121,6 +180,7 @@ def get_policies(session=None, deleted=False):
             all())
 
 
+@db_utils.retry_on_db_error
 def policy_name(name_or_id, session=None):
     session = session or db.get_session()
     try:
@@ -166,9 +226,20 @@ class PolicyRule(model_base.BASE, model_base.HasId, model_base.HasAudit):
         return d
 
 
+@db_utils.retry_on_db_error
 def add_policy_rule(id, policy_name, rule, comment, deleted=False,
                     rule_name="", session=None):
-    session = session or db.get_session()
+    if session:
+        # IMPORTANT: if session provided, do not interrupt existing transaction
+        # with BEGIN which can drop db locks and change desired transaction
+        # boundaries for proper commit and rollback
+        policy_rule = PolicyRule(id, policy_name, rule, comment,
+                                 deleted, rule_name=rule_name)
+        session.add(policy_rule)
+        return policy_rule
+
+    # else
+    session = db.get_session()
     with session.begin(subtransactions=True):
         policy_rule = PolicyRule(id, policy_name, rule, comment,
                                  deleted, rule_name=rule_name)
@@ -176,12 +247,14 @@ def add_policy_rule(id, policy_name, rule, comment, deleted=False,
     return policy_rule
 
 
+@db_utils.retry_on_db_error
 def delete_policy_rule(id, session=None):
     """Specify either the ID or the NAME, and that policy is deleted."""
     session = session or db.get_session()
     return session.query(PolicyRule).filter(PolicyRule.id == id).soft_delete()
 
 
+@db_utils.retry_on_db_error
 def get_policy_rule(id, policy_name, session=None, deleted=False):
     session = session or db.get_session()
     rule_query = (session.query(PolicyRule).
@@ -196,6 +269,7 @@ def get_policy_rule(id, policy_name, session=None, deleted=False):
         pass
 
 
+@db_utils.retry_on_db_error
 def get_policy_rules(policy_name=None, session=None,
                      deleted=False):
     session = session or db.get_session()
