@@ -16,7 +16,7 @@
 
 # Save trace setting
 XTRACE=$(set +o | grep xtrace)
-set +o xtrace
+set -o xtrace
 
 
 # Functions
@@ -55,18 +55,77 @@ function configure_congress {
     cp -r $CONGRESS_DIR/library/* $CONGRESS_LIBRARY_DIR
 
     # Update either configuration file
+    iniset $CONGRESS_CONF DEFAULT bind_host $(ipv6_unquote $SERVICE_LISTEN_ADDRESS)
     iniset $CONGRESS_CONF DEFAULT debug $ENABLE_DEBUG_LOG_LEVEL
     iniset $CONGRESS_CONF DEFAULT auth_strategy $CONGRESS_AUTH_STRATEGY
     iniset $CONGRESS_CONF DEFAULT datasource_sync_period 30
     iniset $CONGRESS_CONF DEFAULT replicated_policy_engine "$CONGRESS_REPLICATED"
-    iniset $CONGRESS_CONF DEFAULT transport_url rabbit://$RABBIT_USERID:$RABBIT_PASSWORD@$RABBIT_HOST:5672
+    iniset $CONGRESS_CONF DEFAULT transport_url rabbit://$RABBIT_USERID:$RABBIT_PASSWORD@$RABBIT_HOST:5672/
     iniset $CONGRESS_CONF database connection `database_connection_url $CONGRESS_DB_NAME`
+    if [ "$ENABLE_CONGRESS_JSON" == "True" ]; then
+        iniset $CONGRESS_CONF json_ingester enable "True"
+        # when the main db is not postgres, the devstack function
+        # database_connection_url_postgresql returns URL with wrong prefix,
+        # so we do a substitution here
+        local db_connection_mysql=`database_connection_url_postgresql $CONGRESS_JSON_DB_NAME`
+        CONGRESS_JSON_DB_CONNECTION_URL=${db_connection_mysql/?*:\/\//postgresql:\/\/}
+        iniset $CONGRESS_CONF json_ingester db_connection $CONGRESS_JSON_DB_CONNECTION_URL
+        iniset $CONGRESS_CONF json_ingester config_path "$CONGRESS_JSON_CONF_DIR"
+        iniset $CONGRESS_CONF json_ingester config_reusables_path "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+
+        # setup json ingester config files
+        if [[ ! -d $CONGRESS_JSON_CONF_DIR ]]; then
+            mkdir $CONGRESS_JSON_CONF_DIR
+        fi
+        echo "keystone_admin_auth_config:" > "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "  type: keystone" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "  config:" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "    project_name: $OS_PROJECT_NAME" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "    username: $OS_USERNAME" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "    password: $OS_PASSWORD" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        echo "    auth_url: `openstack catalog show keystone -f value -c endpoints | sed -n 's/^\s*public: //p' | sed -n '1p'`" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+    fi
 
     _congress_setup_keystone $CONGRESS_CONF keystone_authtoken
+}
 
+function _configure_congress_json_ingester {
+    local endpoint=`openstack catalog show $1 -f value -c endpoints | sed -n 's/^\s*public:\s*//p' | sed -n '1p'`
+    if [[ ! -z $endpoint ]]; then
+        echo "$1_api_endpoint: $endpoint" >> "$CONGRESS_JSON_CONF_REUSABLES_PATH"
+        _copy_congress_json_ingester_config $1
+    fi
+}
+
+function _copy_congress_json_ingester_config {
+    cp $CONGRESS_DIR/etc/sample_json_ingesters/$1.yaml $CONGRESS_JSON_CONF_DIR/
 }
 
 function configure_congress_datasources {
+    if [ "$ENABLE_CONGRESS_JSON" == "True" ]; then
+        _configure_congress_json_ingester cinderv3
+        _configure_congress_json_ingester glance
+        _configure_congress_json_ingester heat
+        _configure_congress_json_ingester keystone
+        _configure_congress_json_ingester magnum
+        _configure_congress_json_ingester masakari
+        _configure_congress_json_ingester mistral
+        _configure_congress_json_ingester neutron
+        _configure_congress_json_ingester nova
+        _configure_congress_json_ingester tacker
+        _configure_congress_json_ingester zun
+        _copy_congress_json_ingester_config monasca
+        _copy_congress_json_ingester_config cve
+        if [ "$CONGRESS_MULTIPROCESS_DEPLOYMENT" == "False" ]; then
+            restart_service devstack@congress.service
+            echo "Waiting for Congress to restart..."
+            if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q -O- http://$CONGRESS_HOST:$CONGRESS_PORT; do sleep 1; done"; then
+                die $LINENO "Congress did not restart"
+            fi
+        else
+            restart_service devstack@congress-datasources.service
+        fi
+    fi
     _configure_service neutron neutronv2
     _configure_service neutron-qos neutronv2_qos
     _configure_service nova nova
@@ -80,6 +139,7 @@ function configure_congress_datasources {
     _configure_service heat heat
     _configure_service aodh aodh
     _configure_service mistral mistral
+    _configure_service tacker tacker
     if [[ $ENABLE_CONGRESS_AGENT == "True" ]] ; then
         _configure_service congress-agent config
     fi
@@ -94,7 +154,7 @@ function _configure_tempest {
     # set correctly due to different env setup scenario, so it is
     # better to set it explicitly here.
     local service
-    local required_services="heat,ironic,aodh,murano,mistral,monasca,neutron-qos"
+    local required_services="heat,ironic,aodh,murano,mistral,monasca,neutron-qos,tacker"
     for service in ${required_services//,/ }; do
         if is_service_enabled $service ; then
             iniset $TEMPEST_CONFIG service_available $service "True"
@@ -102,10 +162,16 @@ function _configure_tempest {
             iniset $TEMPEST_CONFIG service_available $service "False"
         fi
     done
+    iniset $TEMPEST_CONFIG service_available congress "True"
     # Notify tempest if z3 is enabled.
     if [[ $ENABLE_CONGRESS_Z3 == "True" ]] ; then
         iniset $TEMPEST_CONFIG congressz3 enabled "True"
     fi
+
+    # Set feature flags
+    # (remove when Queens no longer supported)
+    iniset $TEMPEST_CONFIG congress-feature-enabled monasca_webhook "True"
+    iniset $TEMPEST_CONFIG congress-feature-enabled vitrage_webhook "True"
 }
 
 function _configure_service {
@@ -146,6 +212,11 @@ function _install_z3 {
         pushd $CONGRESS_Z3_DIR
         z3rel="z3-${USE_Z3_RELEASE}"
         z3file="${z3rel}-x64-${os_VENDOR,,}-${os_RELEASE}"
+        # binary not available for ubuntu-18, so use ubuntu-16 binary instead
+        if [ ${os_VENDOR,,} == "ubuntu" ] && [ ${os_RELEASE} == "18.04" ]; then
+            z3file="${z3rel}-x64-ubuntu-16.04"
+            echo "WARNING: Using ${z3file} binary on ${os_VENDOR,,}-${os_RELEASE} because ${z3rel}-x64-${os_VENDOR,,}-${os_RELEASE} is not available."
+        fi
         url="https://github.com/Z3Prover/z3/releases/download/${z3rel}/${z3file}.zip"
         if [ ! -f "${z3file}.zip" ]; then
             wget "${url}" || true
@@ -220,22 +291,37 @@ function create_congress_accounts {
 # init_congress() - Initialize databases, etc.
 function init_congress {
     recreate_database $CONGRESS_DB_NAME utf8
+    if [ "$ENABLE_CONGRESS_JSON" == "True" ]; then
+        if [ ${DATABASE_TYPE,,} != "postgresql" ]; then
+            # setup separate postgres db if main is not already postgres
+            install_database_postgresql
+            install_database_python_postgresql
+            configure_database_postgresql
+        fi
+        recreate_database_postgresql $CONGRESS_JSON_DB_NAME utf8
+        psql --set=ingester_role="$CONGRESS_JSON_INGESTER_ROLE" \
+             --set=user_role="$CONGRESS_JSON_USER_ROLE" \
+             --set=db_name="$CONGRESS_JSON_DB_NAME" \
+             $CONGRESS_JSON_DB_CONNECTION_URL \
+             -f $CONGRESS_DIR/scripts/jgress/setup_permissions.sql
+    fi
     # Run Congress db migrations
     congress-db-manage --config-file $CONGRESS_CONF upgrade head
 }
 
+function install_congress_pythonclient() {
+# For using non-released client from git branch, need to add
+# LIBS_FROM_GIT=python-congressclient parameter to localrc.
+# Otherwise, congress will install python-congressclient from requirements.
+    if use_library_from_git "python-congressclient"; then
+        git_clone_by_name "python-congressclient"
+        setup_dev_lib "python-congressclient"
+    fi
+}
+
 # install_congress() - install dependency, collect client source and prepare
 function install_congress {
-    # congress requires java so we install it here
-    if is_ubuntu; then
-        install_package default-jre
-    elif is_fedora; then
-        install_package jre
-    else
-        die $LINENO "Congress devstack only supports Debian and Red Hat-based"
-    fi
-    git_clone $CONGRESSCLIENT_REPO $CONGRESSCLIENT_DIR $CONGRESSCLIENT_BRANCH
-    setup_develop $CONGRESSCLIENT_DIR
+    install_congress_pythonclient
 
     if is_service_enabled horizon; then
         _install_congress_dashboard
@@ -296,6 +382,13 @@ function start_congress_service_and_check {
     fi
 }
 
+function _wait_for_congress {
+    echo "Waiting for Congress to start..."
+    if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q -O- http://$CONGRESS_HOST:$CONGRESS_PORT; do sleep 1; done"; then
+        die $LINENO "Congress did not start"
+    fi
+}
+
 
 # stop_congress() - Stop running processes (non-screen)
 function stop_congress {
